/* CD2MP3 PM 1.0 (C) 1998 Samuel Audet <guardia@cam.org> */

#define INCL_PM
#define INCL_DOS
#include <os2.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "prfsam.h"
#include "pmsam.h"
#include "cd2mp3pm.h"
#include "cd2mp3.h"

#include "wav2mp3.h"
#include "cd2wav.h"

#include "dlgwav.h"
#include "dlgmp3.h"
#include "dlgcd.h"

HWND mainHwnd, frameHwnd;

/* notebook page information, see pmsam.h */

NBPAGE nbpage[] =
{
	{wpMP3, "Tab 1 of 4", "~MP3",   DLG_MP3,   FALSE, BKA_MAJOR, 0, 0},
	{wpCD,  "Tab 2 of 4", "~CD",    DLG_CD,    FALSE, BKA_MAJOR, 0, 0},
	{wpWAV, "Tab 3 of 4", "~WAV",   DLG_WAV,   FALSE, BKA_MAJOR, 0, 0},
	{NULL,  "Tab 4 of 4", "~About", DLG_ABOUT, FALSE, BKA_MAJOR, 0, 0}
};

/* some constants so that we don't have to modify code if we add pages */
#define PAGE_COUNT (sizeof( nbpage ) / sizeof( NBPAGE ))
#define PAGE_MP3	  0
#define PAGE_CD	  1
#define PAGE_WAV	  2
#define PAGE_ABOUT  3

/* stuff used during conversion */
ENCODER useEncoder;
GRABBER useGrabber;
int bitrate;
char *wavfilelist;
int wavcount;
int tracklist[256];
int trackcount;
char tempdir[256];
char mp3dir[256];
char basename[256];
char customcd[256];
char custommp3[256];
BOOL hq, lowprio, remove_deselect;
BOOL superGrab;
char cddrive;

/* currently running stuff */
HAPP runningMP3, runningCD;
GRABBER *runningGrabber;
ENCODER *runningEncoder;

/* variable to abort all operations */
BOOL abortEncoding;

/* hApp -> the app that just terminated, or -1 if new */
void doConversion(HAPP hApp, ULONG rc, HWND hwnd)
{
/* structures to keep alive for when a running app terminates */
	static WAV2MP3_END runningMP3end;
	static CD2WAV_END runningCDend;

/* for deselecting a track when finished */
	static int grabbingTrack;
	static int encodingTrack;

/* for removing wav file from the container when finished */
	static char encodingWavFile[512];

/* current wav file in wavfilelist */
	static char *listpos;

/* current track from tracklist */
	static int trackpos;

/* buffers to manage who's doing what */
	static char filetoencode[512], fileencoding[512], filegrabbing[512];

/* to know what just terminated */
	enum {NONE, WAV2MP3, CD2WAV} finishing = NONE;

	if(abortEncoding)
	{
		WinPostMsg(hwnd,WM_COMMAND, MPFROMSHORT(CD2MP3_STOPPED),0);
		return;
	}

	if(hApp == (HAPP) -1)
	{
		listpos = wavfilelist;
		trackpos = 0;

		finishing = WAV2MP3;
		runningMP3 = NULLHANDLE;
		runningCD = NULLHANDLE;

		*encodingWavFile = 0;
		encodingTrack = 0;
		grabbingTrack = 0;

		*filetoencode = 0;
		*fileencoding = 0;
		*filegrabbing = 0;
	}
	else if(hApp == runningMP3)
	{
		if(rc != 0)
		{
			updateError("%s returned with errorlevel %d, aborting MP3 encoding.",runningEncoder->exe, rc);
			WinPostMsg(hwnd,WM_COMMAND, MPFROMSHORT(CD2MP3_STOPPED),0);
			WinTerminateApp(runningCD);
			return;
		}

		wav2mp3_end(&runningMP3end);
		runningMP3 = NULLHANDLE;
		finishing = WAV2MP3;
      if(*encodingWavFile && remove_deselect)
		{
			removeTitleFromContainer(nbpage[PAGE_WAV].hwnd,CT_WAV,encodingWavFile);
			*encodingWavFile = 0;
		}
      if(encodingTrack && remove_deselect)
		{
			CDTRACKRECORD *record;
			record = (CDTRACKRECORD *) enumRecords(nbpage[PAGE_CD].hwnd, CT_TRACK, NULL, CMA_FIRST);
			while(record && record != (CDTRACKRECORD *) -1)
			{
				if(record->track == encodingTrack)
				{
					selectRecord(nbpage[PAGE_CD].hwnd, CT_TRACK, (RECORDCORE *) record, FALSE);
					break;
				}
				record = (CDTRACKRECORD *) enumRecords(nbpage[PAGE_CD].hwnd,CT_TRACK, (RECORDCORE *) record, CMA_NEXT);
			}
			encodingTrack = 0;
		}

		remove(fileencoding); /* harmless for listpos and wavfilelist */
		*fileencoding = 0;
	}
	else if(hApp == runningCD)
	{
		if(rc != 0)
		{
			updateError("%s returned with errorlevel %d, aborting MP3 encoding.",runningGrabber->exe, rc);
			WinPostMsg(hwnd,WM_COMMAND, MPFROMSHORT(CD2MP3_STOPPED),0);
			WinTerminateApp(runningMP3);
			return;
		}
		cd2wav_end(&runningCDend);
		runningCD = NULLHANDLE;
		finishing = CD2WAV;
		strcpy(filetoencode,filegrabbing);
		*filegrabbing = 0;
	}

	/* do new stuff */
	if(listpos && *listpos && finishing != NONE)
	{
		WAV2MP3_IN input;
		input.hwnd = hwnd;
		runningEncoder = &useEncoder;
		input.encoder = &useEncoder;
		input.wavfile = listpos;
		input.mp3dir = mp3dir;
		input.custom = custommp3;
		input.bitrate = bitrate;
		input.hq = hq;
		input.lowprio = lowprio;
		runningMP3 = wav2mp3(&input, &runningMP3end);
		strcpy(encodingWavFile,listpos);
		listpos = strchr(listpos,0)+1;
		if(!listpos)
			free(wavfilelist);
	}
	else
	{
		if( (finishing == CD2WAV && !runningMP3) ||
			 (superGrab && finishing == WAV2MP3 && *filetoencode) )
		{
			WAV2MP3_IN input;

			strcpy(fileencoding,filetoencode);
			*filetoencode = 0;

			input.hwnd = hwnd;
			runningEncoder = &useEncoder;
			input.encoder = &useEncoder;
			input.wavfile = fileencoding;
			input.mp3dir = mp3dir;
			input.custom = custommp3;
			input.bitrate = bitrate;
			input.hq = hq;
			input.lowprio = lowprio;
			runningMP3 = wav2mp3(&input, &runningMP3end);
			encodingTrack = grabbingTrack;
		}

		if(trackpos < trackcount && ( (finishing == WAV2MP3 && !runningCD) ||
			(superGrab && finishing == CD2WAV && !*filetoencode) ) )
		{
			CD2WAV_IN input;
			sprintf(filegrabbing, "%s%s_%.2d.wav", tempdir, basename, tracklist[trackpos]);

			input.hwnd = hwnd;
			runningGrabber = &useGrabber;
			input.grabber = &useGrabber;
			input.wavname = filegrabbing;
			input.custom = customcd;
			input.track = tracklist[trackpos];
			grabbingTrack = input.track;
			if(superGrab)
				input.lowprio = lowprio;
			else
				input.lowprio = FALSE;
			input.cddrive = cddrive;
			runningCD = cd2wav(&input, &runningCDend);

			trackpos++;
		}

		if(trackpos >= trackcount && !runningCD && !runningMP3 && !*filetoencode)
			WinPostMsg(hwnd,WM_COMMAND, MPFROMSHORT(CD2MP3_STOPPED),0);
	}
}

void beginConverter(HWND hwnd)
{
	int i;
	char buffer[512];
	CDTRACKRECORD *selectedCDRecord;
	RECORDCORE *WAVRecord;

	/* looking at MP3 settings */

	hq = getCheck(nbpage[PAGE_MP3].hwnd, CB_HQ);
	lowprio = getCheck(nbpage[PAGE_MP3].hwnd, CB_LOWPRIO);
	getText(nbpage[PAGE_MP3].hwnd, EF_MP3DIR, mp3dir, sizeof(mp3dir));
	if(*mp3dir && *(strchr(mp3dir,0)-1) != '\\') strcat(mp3dir,"\\");
	getText(nbpage[PAGE_MP3].hwnd, EF_CUSTOMMP3, custommp3, sizeof(custommp3));

	*useEncoder.id = 0;
	getText(WinWindowFromID(nbpage[PAGE_MP3].hwnd, CB_ENCODER), CBID_EDIT, buffer, sizeof(buffer));
	for(i = 0; i < countEncoders; i++)
	{
		if(!strcmp(encoders[i].id,buffer))
		{
			useEncoder = encoders[i];
			break;
		}
	}
	if(!*useEncoder.id)
		updateError("No valid Encoder chosen.");

	getText(WinWindowFromID(nbpage[PAGE_MP3].hwnd, CB_BITRATE), CBID_EDIT, buffer, sizeof(buffer));
	bitrate = atoi(buffer);

   remove_deselect = getCheck(nbpage[PAGE_MP3].hwnd, CB_REMOVE);

	/* looking at CD settings */

	getText(nbpage[PAGE_CD].hwnd, EF_BASENAME, basename, sizeof(basename));
	getText(nbpage[PAGE_CD].hwnd, EF_TEMPDIR, tempdir, sizeof(tempdir));
	if(*tempdir && *(strchr(tempdir,0)-1) != '\\') strcat(tempdir,"\\");
	getText(nbpage[PAGE_CD].hwnd, EF_CUSTOMCD, customcd, sizeof(customcd));
	getText(WinWindowFromID(nbpage[PAGE_CD].hwnd, CB_DRIVE), CBID_EDIT, buffer, sizeof(buffer));
	cddrive = buffer[0];

	*useGrabber.id = 0;
	getText(WinWindowFromID(nbpage[PAGE_CD].hwnd, CB_GRABBER), CBID_EDIT, buffer, sizeof(buffer));
	for(i = 0; i < countGrabbers; i++)
	{
		if(!strcmp(grabbers[i].id,buffer))
		{
			useGrabber = grabbers[i];
			break;
		}
	}
	if(!*useGrabber.id)
		updateError("No valid Grabber chosen.");

	trackcount = 0;
	selectedCDRecord = (CDTRACKRECORD *) searchRecords(nbpage[PAGE_CD].hwnd, CT_TRACK,
							 (RECORDCORE *) CMA_FIRST, CRA_SELECTED);
	while(selectedCDRecord && selectedCDRecord != (CDTRACKRECORD *) -1)
	{
		tracklist[trackcount++] = selectedCDRecord->track;
		selectedCDRecord = (CDTRACKRECORD *)searchRecords(nbpage[PAGE_CD].hwnd, CT_TRACK,
								 (RECORDCORE *) selectedCDRecord, CRA_SELECTED);
	}

	superGrab = getCheck(nbpage[PAGE_CD].hwnd, CB_SUPERGRAB);

	/* looking at WAV settings */
	makeWavList(nbpage[PAGE_WAV].hwnd, CT_WAV, &wavfilelist);

	abortEncoding = FALSE;
	doConversion(-1, 0, hwnd);
}

static MRESULT processCommand(HWND hwnd,MPARAM mp1,MPARAM mp2)
{
	switch(SHORT1FROMMP(mp1))
	{
		case PB_CONVERT:
			disable(hwnd,PB_CONVERT);
			enable(hwnd,PB_STOP);
			beginConverter(hwnd);
			break;

		case PB_STOP:
			abortEncoding = TRUE;
			if(runningCD)
				if(!WinTerminateApp(runningCD))
					updateError("Could not kill %s, sorry.",runningGrabber->exe);
			if(runningMP3)
				if(!WinTerminateApp(runningMP3))
					updateError("Could not kill %s, sorry.",runningEncoder->exe);
			break;

		case CD2MP3_STOPPED:
			enable(hwnd,PB_CONVERT);
			disable(hwnd,PB_STOP);
			break;
	}
	return 0;
}

static MRESULT processControl(HWND hwnd,MPARAM mp1,MPARAM mp2)
{
#if 0
	switch(SHORT1FROMMP(mp1))
	{
	}
#endif
	return 0;
}

SHORT NBy; /* reserve some client window space for buttons and errors */

static MRESULT processInitdlg(HWND hwnd,MPARAM mp1,MPARAM mp2)
{
	int i;
	HWND hwndNB;
	ULONG xspace;
	POINTL size[2];
	LONG color;
	ACCELTABLE *acceltable;
	ULONG tablesize;
	HACCEL hAccel;
	char *font;

	/* those fuckin "/$%@* accElERATor (drunk) tables.. removing shift-f10
		and shift-f9 for PROPER processing further along in notebook dialogs */
	hAccel = WinQueryAccelTable(hab, frameHwnd);
	tablesize = WinCopyAccelTable(hAccel,NULL,0);
	acceltable = alloca(tablesize);
	WinCopyAccelTable(hAccel,acceltable,tablesize);

	for(i = 0; i < acceltable->cAccel; i++)
	{
		ACCEL *accelentry = &acceltable->aaccel[i];

		if((accelentry->fs & KC_SHIFT) && accelentry->key == VK_F10)
			accelentry->fs = accelentry->key = accelentry->cmd = 0;

		else if((accelentry->fs & KC_SHIFT) && accelentry->key == VK_F9)
			accelentry->fs = accelentry->key = accelentry->cmd = 0;
	}

	WinSetAccelTable(hab, NULLHANDLE, frameHwnd);
	hAccel = WinCreateAccelTable(hab, acceltable);
	WinSetAccelTable(hab, hAccel, frameHwnd);

   /* creating and loading all the nice stuff */
	hwndNB = createNotebook(hwnd, nbpage, PAGE_COUNT);
	if(!hwndNB) return (MRESULT) 1;

	if(!loadNotebookDlg(hwndNB, nbpage, PAGE_COUNT))
		return (MRESULT) 1;

	size[0].x = 26; size[0].y = 13;
	size[1].x = 34; size[1].y = 13;
	WinMapDlgPoints(HWND_DESKTOP, size, 2, TRUE);
	NBy = size[0].y+3;

	xspace = 5 + size[1].x + 5 + size[0].x + 5;
	initError(hwnd,xspace,0,400,NBy);

	xspace = 5 + size[1].x + 5;
	WinCreateWindow(hwnd, WC_BUTTON, "~Stop",
						 BS_PUSHBUTTON | WS_TABSTOP | WS_VISIBLE,
						 xspace, 0, size[0].x, size[0].y,
						 hwnd, HWND_TOP, PB_STOP, NULL, NULL);

	xspace = 5;
	WinCreateWindow(hwnd, WC_BUTTON, "~Convert",
						 BS_PUSHBUTTON | WS_TABSTOP | WS_VISIBLE,
						 xspace, 0, size[1].x, size[1].y,
						 hwnd, HWND_TOP, PB_CONVERT, NULL, NULL);

	enable(hwnd,PB_CONVERT);
	disable(hwnd,PB_STOP);

	color = SYSCLR_FIELDBACKGROUND;
	WinSetPresParam(hwnd, PP_BACKGROUNDCOLORINDEX, sizeof(color), &color);

	if(isWarpSans())
		font = "9.WarpSans";
	else
		font = "8.Helv";

	WinSetPresParam(hwnd, PP_FONTNAMESIZE, strlen(font)+1, font);
	for(i = 0; i < PAGE_COUNT; i++)
		WinSetPresParam(nbpage[i].hwnd, PP_FONTNAMESIZE, strlen(font)+1, font);

	return 0;
}

MRESULT EXPENTRY wpMain(HWND hwnd,ULONG msg,MPARAM mp1,MPARAM mp2)
{
	switch (msg)
	{
		case WM_CREATE:
			return processInitdlg(hwnd,mp1,mp2);
		case WM_COMMAND:
			return processCommand(hwnd,mp1,mp2);
		case WM_CONTROL:
			return processControl(hwnd,mp1,mp2);
		case WM_ERASEBACKGROUND:
			WinFillRect(LONGFROMMP(mp1), PVOIDFROMMP(mp2), SYSCLR_FIELDBACKGROUND);
			return (MRESULT) FALSE;
#if 0
		case WM_ERASEBACKGROUND:
		{
			RECTL *rect = PVOIDFROMMP(mp2);
			POINTL pos;
			HBITMAP hbm;
			BITMAPINFOHEADER info;

			hbm = GpiLoadBitmap(HWNDFROMMP(mp1),0,BM_BACKGROUND,0,0);
			info.cbFix = sizeof(info);
			if(GpiQueryBitmapParameters(hbm,&info))
			{
				pos.x = 0;
				while(pos.x < rect->xRight)
				{
					pos.y = 0;
					while(pos.y < rect->yTop)
					{
						WinDrawBitmap(HWNDFROMMP(mp1), hbm, NULL, &pos, 0, 0, DBM_NORMAL);
						pos.y += info.cy;
					}
					pos.x += info.cx;
				}
			}
			return 0;
		}
#endif
		case WM_PAINT:
			WinDefWindowProc(hwnd,msg,mp1,mp2);
/* ?? */
			return 0;

		case WM_SIZE:

				// Size the notebook with the client window

				WinSetWindowPos( WinWindowFromID( hwnd, FID_CLIENT ), 0, 0, NBy,
									  SHORT1FROMMP( mp2 ), SHORT2FROMMP( mp2 ) - NBy,
									  SWP_SIZE | SWP_SHOW | SWP_MOVE);

				break;

		case WM_CHAR:
			doControlNavigation(hwnd,mp1,mp2);
			break;

		case WM_APPTERMINATENOTIFY:
			doConversion(LONGFROMMP(mp1), LONGFROMMP(mp2), hwnd);
			return 0;

		case WM_CLOSE:
			WinSendMsg(nbpage[PAGE_MP3].hwnd,msg,mp1,mp2);
			WinSendMsg(nbpage[PAGE_CD].hwnd,msg,mp1,mp2);
			WinSendMsg(nbpage[PAGE_WAV].hwnd,msg,mp1,mp2);
			WinPostMsg(hwnd,WM_QUIT,0,0);
			return 0;
	}

	return WinDefWindowProc(hwnd,msg,mp1,mp2);
}

int main(int argc, char *argv[])
{
	ULONG frameFlags = FCF_TITLEBAR | FCF_SYSMENU | FCF_MINBUTTON |/* FCF_MAXBUTTON | */
							 FCF_SIZEBORDER  |/* FCF_MENU | FCF_ACCELTABLE | */ FCF_ICON |
						 /* FCF_SHELLPOSITION | */ FCF_TASKLIST /*| FCF_DLGBORDER*/;

	if(initPM() &&
		(frameHwnd = createClientFrame(&mainHwnd, "CD2MP3 PM", &frameFlags,
												 wpMain, "CD2MP3 PM")))
	{
		if(!loadPosition(frameHwnd, INIFILE))
		{
			POINTL size = {350, 200};
			WinMapDlgPoints(HWND_DESKTOP, &size, 1, TRUE);

			WinSetWindowPos( frameHwnd, NULLHANDLE,
								  30, 30, size.x, size.y,
								  SWP_SIZE | SWP_MOVE | SWP_SHOW);
		}

		runPM();

		savePosition(frameHwnd, INIFILE);

		closePM();

		return 0;
	}

	return 1;
}

/* for external use */
int bitRateChanged = 0;
USHORT getBitRate(void)
{
	char buffer[8];
	getText(WinWindowFromID(nbpage[PAGE_MP3].hwnd, CB_BITRATE), CBID_EDIT, buffer, sizeof(buffer));
	return atoi(buffer);
}
