/* CD2MP3 PM 1.0 (C) 1998 Samuel Audet <guardia@cam.org> */

#define INCL_PM
#define INCL_DOS
#include <os2.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "prfsam.h"
#include "pmsam.h"
#include "cd2mp3pm.h"
#include "cd2mp3.h"

#include "readcd.h"
#include "miscsam.h"
static PFNWP wpCT;

/* keeps a list of loaded encoders and grabbers, and their settings */
GRABBER *grabbers;
int countGrabbers;

/* container icons */
static HPOINTER dataIco, trackIco;

/* for WM_ENDMENU */
static removeSourceEmphasisInfo sourceEmphasisInfo;
static BOOL miniIcons = 0;

static BOOL loadConfig(char *file, GRABBER **grabbers, int *countGrabbers)
{
	FILE *config;
	config = fopen(file,"r");

	if(config)
	{
		char buffer[512], param[256], value[256], *comment;
		int e;

		e = 0;
		while(fgets(buffer,sizeof(buffer),config))
		{
			uncomment(buffer);
			sscanf(buffer,"%[^=]=%[^=]",param,value);
			strip(param);

			if(!stricmp(param,"grabberid")) e++;
		}

		*grabbers = (GRABBER *) calloc(e, sizeof(GRABBER));

		rewind(config);

		e = -1;
		while(fgets(buffer,sizeof(buffer),config))
		{
			uncomment(buffer);
			sscanf(buffer,"%[^=]=%[^=]",param,value);
			strip(param);
			strip(value);

			if(!stricmp(param,"grabberid"))
			{
				e++;
				strcpy((*grabbers)[e].id, value);
			}
			else if(e >=0)
			{
				if(!stricmp(param,"grabberexe"))
					strcpy((*grabbers)[e].exe, value);
				else if(!stricmp(param,"grabbertype"))
				{
					if(!stricmp(value,"win32os2"))
						(*grabbers)[e].type = WIN32OS2;
					else if(!stricmp(value,"vio"))
						(*grabbers)[e].type = VIO;
					else if(!stricmp(value,"vdm"))
						(*grabbers)[e].type = VDM;
				}
				else if(!stricmp(param,"grabberin"))
					strcpy((*grabbers)[e].input, value);
				else if(!stricmp(param,"grabberout"))
					strcpy((*grabbers)[e].output, value);
				else if(!stricmp(param,"grabbermangling"))
					strcpy((*grabbers)[e].mangling, value);
				else if(!stricmp(param,"grabberbefore"))
					strcpy((*grabbers)[e].before, value);
				else if(!stricmp(param,"grabberafter"))
					strcpy((*grabbers)[e].after, value);
				else if(!stricmp(param,"grabberacceptsoutput"))
				{
					if(!stricmp(value,"true"))
						(*grabbers)[e].acceptsOutput = TRUE;
					else
						(*grabbers)[e].acceptsOutput = FALSE;
				}
				else if(!stricmp(param,"grabbernodrive"))
				{
					if(!stricmp(value,"true"))
						(*grabbers)[e].noDrive = TRUE;
					else
						(*grabbers)[e].noDrive = FALSE;
				}
			}
		}
		*countGrabbers = e+1;

		fclose(config);
		return TRUE;
	}
	return FALSE;
}

static void loadIni(HWND hwnd)
{
	char buffer[512];
   CNRINFO cnrInfo;
	HINI inifile;

	inifile = openProfile(INIFILE);

   if(inifile)
   {
      if(readProfile(inifile,"CD","Grabber",buffer,sizeof(buffer), TRUE))
         setText(WinWindowFromID(hwnd, CB_GRABBER), CBID_EDIT, buffer);
      if(readProfile(inifile,"CD","Drive",buffer,sizeof(buffer), TRUE))
         setText(WinWindowFromID(hwnd, CB_DRIVE), CBID_EDIT, buffer);
      if(readProfile(inifile,"CD","Basename",buffer,sizeof(buffer), TRUE))
         setText(hwnd, EF_BASENAME, buffer);
      if(readProfile(inifile,"CD","TempDir",buffer,sizeof(buffer), TRUE))
         setText(hwnd, EF_TEMPDIR, buffer);
      if(readProfile(inifile,"CD","Custom",buffer,sizeof(buffer), TRUE))
         setText(hwnd, EF_CUSTOMCD, buffer);
      if(readProfile(inifile,"CD","SuperGrab",buffer,1, FALSE))
         setCheck(hwnd, CB_SUPERGRAB, *buffer);

      if(readProfile(inifile,"CD","ContainerInfo",&cnrInfo,sizeof(cnrInfo), FALSE))
         WinSendDlgItemMsg(hwnd,CT_TRACK,CM_SETCNRINFO, MPFROMP(&cnrInfo),
            MPFROMLONG(CMA_XVERTSPLITBAR | CMA_FLWINDOWATTR));

      miniIcons = (cnrInfo.flWindowAttr & CV_MINI) ? TRUE : FALSE;

      closeProfile(inifile);
   }
}

static void saveIni(HWND hwnd)
{
	char buffer[512];
   CNRINFO cnrInfo;
   HINI inifile;

	inifile = openProfile(INIFILE);

   if(inifile)
   {
      getText(WinWindowFromID(hwnd, CB_GRABBER), CBID_EDIT, buffer, sizeof(buffer));
      writeProfile(inifile,"CD","Grabber", buffer,0);
      getText(WinWindowFromID(hwnd, CB_DRIVE), CBID_EDIT, buffer, sizeof(buffer));
      writeProfile(inifile,"CD","Drive", buffer,0);
      getText(hwnd, EF_BASENAME, buffer, sizeof(buffer));
      writeProfile(inifile,"CD","Basename", buffer,0);
      getText(hwnd, EF_TEMPDIR, buffer, sizeof(buffer));
      writeProfile(inifile,"CD","TempDir", buffer,0);
      getText(hwnd, EF_CUSTOMCD, buffer, sizeof(buffer));
      writeProfile(inifile,"CD","Custom", buffer,0);
      *buffer = getCheck(hwnd, CB_SUPERGRAB);
      writeProfile(inifile,"CD","SuperGrab", buffer, 1);

      cnrInfo.cb = sizeof(cnrInfo);
      WinSendDlgItemMsg(hwnd,CT_TRACK,CM_QUERYCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(sizeof(cnrInfo)));
      writeProfile(inifile,"CD","ContainerInfo", &cnrInfo, sizeof(cnrInfo));

      closeProfile(inifile);
   }
}


/* refresh MP3 size with new bitrate */
static void refreshFieldInfo(HWND hwnd, LONG id)
{
   CDTRACKRECORD *record;
   USHORT bitrate = getBitRate();

   record = (CDTRACKRECORD *) enumRecords(hwnd, id, NULL, CMA_FIRST);
   while(record && record != (CDTRACKRECORD *) -1)
   {
      if(!bitrate)
         record->mp3size = 0;
      else if(!record->data)
         record->mp3size = record->size /
            ((record->channels*2*44100) / ((double) getBitRate() * 1000/8)) + 0.5;
      record = (CDTRACKRECORD *) enumRecords(hwnd, id, (RECORDCORE *) record, CMA_NEXT);
   }

   WinSendDlgItemMsg(hwnd, id, CM_INVALIDATEDETAILFIELDINFO,0,0);
}


static MRESULT processControl(HWND hwnd,MPARAM mp1,MPARAM mp2)
{
	switch(SHORT2FROMMP(mp1))
	{
		case CN_CONTEXTMENU:
		{
			RECORDCORE *record = PVOIDFROMMP(mp2);

			processPopUp(hwnd, CT_TRACK, record,
            PUM_CDRECORD, PUM_CDCONTAINER, &sourceEmphasisInfo);

         WinSendMsg(sourceEmphasisInfo.PUMHwnd, MM_SETITEMATTR,
            MPFROM2SHORT(IDM_MINIICONS, FALSE),
            MPFROM2SHORT(MIA_CHECKED, miniIcons ? MIA_CHECKED : 0));

			return 0;
		}
	}

	return 0;
}


static MRESULT processCommand(HWND hwnd,MPARAM mp1,MPARAM mp2)
{
	switch(SHORT1FROMMP(mp1))
	{
		case IDM_ICONVIEW:
		{
			CNRINFO cnrInfo;
			cnrInfo.cb = sizeof(cnrInfo);
         WinSendDlgItemMsg(hwnd,CT_TRACK,CM_QUERYCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(sizeof(cnrInfo)));
         cnrInfo.flWindowAttr |= CV_ICON;
         cnrInfo.flWindowAttr &= ~CV_NAME & ~CV_DETAIL & ~CV_TREE;
			WinSendDlgItemMsg(hwnd,CT_TRACK,CM_SETCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(CMA_FLWINDOWATTR));
			return 0;
		}
		case IDM_NAMEVIEW:
		{
			CNRINFO cnrInfo;
			cnrInfo.cb = sizeof(cnrInfo);
         WinSendDlgItemMsg(hwnd,CT_TRACK,CM_QUERYCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(sizeof(cnrInfo)));
         cnrInfo.flWindowAttr |= CV_NAME | CV_FLOW;
         cnrInfo.flWindowAttr &= ~CV_ICON & ~CV_DETAIL & ~CV_TREE;
			WinSendDlgItemMsg(hwnd,CT_TRACK,CM_SETCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(CMA_FLWINDOWATTR));
			return 0;
		}
		case IDM_DETAILVIEW:
		{
			CNRINFO cnrInfo;
			cnrInfo.cb = sizeof(cnrInfo);
         WinSendDlgItemMsg(hwnd,CT_TRACK,CM_QUERYCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(sizeof(cnrInfo)));
         cnrInfo.flWindowAttr |= CV_DETAIL | CA_DETAILSVIEWTITLES;
         cnrInfo.flWindowAttr &= ~CV_ICON & ~CV_NAME & ~CV_TREE;
			WinSendDlgItemMsg(hwnd,CT_TRACK,CM_SETCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(CMA_FLWINDOWATTR));
			return 0;
		}
      case IDM_MINIICONS:
      {
         CNRINFO cnrInfo;
         miniIcons = !miniIcons;

			cnrInfo.cb = sizeof(cnrInfo);
         WinSendDlgItemMsg(hwnd,CT_TRACK,CM_QUERYCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(sizeof(cnrInfo)));
         cnrInfo.flWindowAttr = (miniIcons) ? (cnrInfo.flWindowAttr | CV_MINI) : (cnrInfo.flWindowAttr & ~CV_MINI);
         WinSendDlgItemMsg(hwnd,CT_TRACK,CM_SETCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(CMA_FLWINDOWATTR));
			return 0;
      }

      case IDM_SELECTALL:
         selectAllRecords(hwnd, CT_TRACK, TRUE);
         return 0;
      case IDM_DESELECTALL:
         selectAllRecords(hwnd, CT_TRACK, FALSE);
         return 0;

      case PB_CDPLAY:
         sourceEmphasisInfo.sourceRecord =
            searchRecords(hwnd, CT_TRACK, (RECORDCORE *) CMA_FIRST, CRA_CURSORED);
      case IDM_PLAY:
      {
         CDTRACKRECORD *record;
         char buffer[512], drive[4];
			HFILE hDrive;
         record = (CDTRACKRECORD *) sourceEmphasisInfo.sourceRecord;

         getText(WinWindowFromID(hwnd, CB_DRIVE), CBID_EDIT, buffer, sizeof(buffer));
			drive[0] = buffer[0];
			drive[1] = ':';
			drive[2] = 0;

         hDrive = openCD(drive);
         if(hDrive)
         {
            stopCD(hDrive);
            playTrack(hDrive, record->track);
            closeCD(hDrive);
         }
         return 0;
      }

      case PB_CDSTOP:
      case IDM_STOP:
      {
         char buffer[512], drive[4];
			HFILE hDrive;

         getText(WinWindowFromID(hwnd, CB_DRIVE), CBID_EDIT, buffer, sizeof(buffer));
			drive[0] = buffer[0];
			drive[1] = ':';
			drive[2] = 0;

         hDrive = openCD(drive);
         if(hDrive)
         {
            stopCD(hDrive);
            closeCD(hDrive);
         }
         return 0;
      }

		case PB_REFRESH:
		case IDM_REFRESH:
		{
			CDTRACKRECORD *record;
			CDTRACKRECORD *firstRecord, *posRecord;
			RECORDINSERT recordInfo;
			int count, i;
			CDAUDIODISKINFODATA CDInfo;
			CDTRACKINFO trackInfo;
			char buffer[512], drive[4];
			HFILE hDrive;
         USHORT bitrate = getBitRate();

			/* delete all current records */
			record = (CDTRACKRECORD *) enumRecords(hwnd, CT_TRACK, NULL, CMA_FIRST);
			while(record && record != (CDTRACKRECORD *) -1)
			{
				free(record->record.pszIcon);
				record = (CDTRACKRECORD *) enumRecords(hwnd, CT_TRACK, (RECORDCORE *) record, CMA_NEXT);
			}

			removeRecords(hwnd, CT_TRACK, NULL, 0);

			/* find the number of CD tracks */
			getText(WinWindowFromID(hwnd, CB_DRIVE), CBID_EDIT, buffer, sizeof(buffer));
			drive[0] = buffer[0];
			drive[1] = ':';
			drive[2] = 0;

			hDrive = openCD(drive);

			if(!hDrive)
			{
				updateError("Could not open drive %s",drive);
				return 0;
			}

			getCDInfo(hDrive, &CDInfo);

			count = CDInfo.lastTrack - CDInfo.firstTrack + 1;

			/* allocate and fill container with CD Track Info */
			firstRecord = posRecord = (CDTRACKRECORD *)
				allocaRecords(hwnd, CT_TRACK, count,
				sizeof(CDTRACKRECORD) - sizeof(RECORDCORE));

			for (i = CDInfo.firstTrack; i <= CDInfo.lastTrack; i++)
			{
            /* loading info fields */
				getTrackInfo(hDrive, i, &trackInfo);

            if(trackInfo.data)
            {
               strcat(posRecord->time,"N/A");
               strcat(posRecord->type,"Data");
               posRecord->mp3size = 0;
               posRecord->record.hptrIcon = dataIco;
               posRecord->record.hptrMiniIcon = 0; /* ask God */
            }
            else
            {
               sprintf(posRecord->time,"%d:%.2d",trackInfo.length.minute,
                                                 trackInfo.length.second);
               strcat(posRecord->type,"Audio");
               if(bitrate)
                  posRecord->mp3size = trackInfo.size /
                     ((trackInfo.channels*2*44100) / ((double) bitrate * 1000/8)) + 0.5;
               else
                  posRecord->mp3size = 0;
               posRecord->record.hptrIcon = trackIco;
               posRecord->record.hptrMiniIcon = 0; /* ask God */
            }

            /* kludge */
            posRecord->timepointer = posRecord->time;
            posRecord->typepointer = posRecord->type;

				posRecord->track = i;
            posRecord->size = trackInfo.size;
            posRecord->channels = trackInfo.channels;
            posRecord->data = trackInfo.data;

            getText(hwnd,EF_BASENAME, buffer, sizeof(buffer));
            sprintf(buffer, "%s_%.2d", buffer, i);
				posRecord->record.pszIcon = posRecord->record.pszText =
				posRecord->record.pszName = posRecord->record.pszTree =
					malloc(strlen(buffer)+1);
				strcpy(posRecord->record.pszIcon,buffer);

            /* "next!" */
				posRecord = (CDTRACKRECORD *) posRecord->record.preccNextRecord;
			}

			closeCD(hDrive);

			recordInfo.cb = sizeof(recordInfo);
			recordInfo.pRecordOrder = (RECORDCORE *) CMA_END;
			recordInfo.pRecordParent = NULL;
			recordInfo.fInvalidateRecord = TRUE;
			recordInfo.zOrder = CMA_TOP;
			recordInfo.cRecordsInsert = count;

			insertRecords(hwnd, CT_TRACK, (RECORDCORE *) firstRecord, &recordInfo);

			return 0;
		}
	}
	return 0;
}


MRESULT EXPENTRY wpCTTrack(HWND hwnd,ULONG msg,MPARAM mp1,MPARAM mp2)
{
	switch(msg)
	{
		case WM_CHAR:
		{
			SHORT fsflags = SHORT1FROMMP(mp1);
			SHORT usvkey = SHORT2FROMMP(mp2);
			if(fsflags & KC_KEYUP) break;

			if((fsflags & KC_SHIFT) && (usvkey == VK_F10))
			{
            RECORDCORE *record;
            record = searchRecords(hwnd, 0, (RECORDCORE *) CMA_FIRST, CRA_SELECTED);

            processPopUp(WinQueryWindow(hwnd, QW_PARENT), WinQueryWindowUShort(hwnd,QWS_ID),
               record, PUM_CDRECORD, PUM_CDCONTAINER, &sourceEmphasisInfo);

            WinSendMsg(sourceEmphasisInfo.PUMHwnd, MM_SETITEMATTR,
               MPFROM2SHORT(IDM_MINIICONS, FALSE),
               MPFROM2SHORT(MIA_CHECKED, miniIcons ? MIA_CHECKED : 0));
				return 0;
			}

         else if((fsflags & KC_SHIFT) && (usvkey == VK_F9))
         {
            CNREDITDATA editdata;
            CNRINFO cnrInfo;

            WinSendMsg(hwnd,CM_QUERYCNRINFO, MPFROMP(&cnrInfo),MPFROMLONG(sizeof(cnrInfo)));

            editdata.cb = sizeof(editdata);
            editdata.hwndCnr = hwnd;
            editdata.pRecord = searchRecords(hwnd, 0, (RECORDCORE *) CMA_FIRST, CRA_CURSORED);
            editdata.ppszText = NULL;
            editdata.cbText = 0;

            if(cnrInfo.flWindowAttr & CV_DETAIL)
            {
               editdata.pFieldInfo = WinSendMsg(hwnd, CM_QUERYDETAILFIELDINFO, MPFROMP(NULL), MPFROMSHORT(CMA_FIRST));
               editdata.pFieldInfo = WinSendMsg(hwnd, CM_QUERYDETAILFIELDINFO, MPFROMP(editdata.pFieldInfo), MPFROMSHORT(CMA_NEXT));
               editdata.id = CID_LEFTDVWND;
            }
            else
            {
               editdata.pFieldInfo = NULL;
               editdata.id = WinQueryWindowUShort(hwnd,QWS_ID);
            }

            WinSendMsg(hwnd,CM_OPENEDIT,MPFROMP(&editdata),0);

            return 0;
         }

			break;
		}
	}

	return wpCT( hwnd, msg, mp1, mp2 );
}


MRESULT EXPENTRY wpCD(HWND hwnd,ULONG msg,MPARAM mp1,MPARAM mp2)
{
	switch (msg)
	{
		case WM_INITDLG:
		{
			ULONG rc, i;
			HFILE CDDevice;
			ULONG action;
			ULONG len;
			struct
			{
				USHORT CountCD;
				USHORT FirstCD;
			} CDInfo;
			FIELDINFO *firstFieldInfo, *fieldInfo, *splitFieldInfo;
			FIELDINFOINSERT fieldInfoInsert;
			CNRINFO cnrInfo;
			cnrInfo.cb = sizeof(cnrInfo);

         firstFieldInfo = fieldInfo = allocaFieldInfo(hwnd, CT_TRACK, 7);
			fieldInfo->flData = CFA_BITMAPORICON | CFA_HORZSEPARATOR | CFA_SEPARATOR;
			fieldInfo->flTitle = CFA_CENTER;
			fieldInfo->pTitleData = "Icon";
         fieldInfo->offStruct = FIELDOFFSET(RECORDCORE,hptrIcon);

			fieldInfo = fieldInfo->pNextFieldInfo;
			fieldInfo->flData = CFA_STRING | CFA_HORZSEPARATOR | CFA_SEPARATOR;
			fieldInfo->flTitle = CFA_CENTER;
			fieldInfo->pTitleData = "Title";
			fieldInfo->offStruct = FIELDOFFSET(RECORDCORE,pszIcon);

			cnrInfo.pFieldInfoLast = fieldInfo;

			fieldInfo = fieldInfo->pNextFieldInfo;
			fieldInfo->flData = CFA_ULONG | CFA_HORZSEPARATOR | CFA_SEPARATOR;
			fieldInfo->flTitle = CFA_CENTER;
			fieldInfo->pTitleData = "Track #";
			fieldInfo->offStruct = FIELDOFFSET(CDTRACKRECORD,track);

         fieldInfo = fieldInfo->pNextFieldInfo;
         fieldInfo->flData = CFA_STRING | CFA_HORZSEPARATOR | CFA_SEPARATOR | CFA_FIREADONLY;
			fieldInfo->flTitle = CFA_CENTER;
         fieldInfo->pTitleData = "Type";
         fieldInfo->offStruct = FIELDOFFSET(CDTRACKRECORD,typepointer);

         fieldInfo = fieldInfo->pNextFieldInfo;
         fieldInfo->flData = CFA_STRING | CFA_HORZSEPARATOR | CFA_SEPARATOR | CFA_FIREADONLY;
			fieldInfo->flTitle = CFA_CENTER;
         fieldInfo->pTitleData = "Time";
         fieldInfo->offStruct = FIELDOFFSET(CDTRACKRECORD,timepointer);

			fieldInfo = fieldInfo->pNextFieldInfo;
			fieldInfo->flData = CFA_ULONG | CFA_HORZSEPARATOR | CFA_SEPARATOR;
			fieldInfo->flTitle = CFA_CENTER;
			fieldInfo->pTitleData = "Size";
			fieldInfo->offStruct = FIELDOFFSET(CDTRACKRECORD,size);

         fieldInfo = fieldInfo->pNextFieldInfo;
			fieldInfo->flData = CFA_ULONG | CFA_HORZSEPARATOR | CFA_SEPARATOR;
			fieldInfo->flTitle = CFA_CENTER;
         fieldInfo->pTitleData = "MP3 Size";
         fieldInfo->offStruct = FIELDOFFSET(CDTRACKRECORD,mp3size);

			fieldInfoInsert.cb = sizeof(fieldInfoInsert);
			fieldInfoInsert.pFieldInfoOrder = (FIELDINFO *) CMA_FIRST;
			fieldInfoInsert.fInvalidateFieldInfo = TRUE;
         fieldInfoInsert.cFieldInfoInsert = 7;

			insertFieldInfo(hwnd, CT_TRACK, firstFieldInfo, &fieldInfoInsert);

			cnrInfo.xVertSplitbar = 100;
			cnrInfo.flWindowAttr = CV_DETAIL | CA_DETAILSVIEWTITLES;
			WinSendDlgItemMsg(hwnd,CT_TRACK,CM_SETCNRINFO, MPFROMP(&cnrInfo),
				MPFROMLONG(CMA_PFIELDINFOLAST | CMA_XVERTSPLITBAR | CMA_FLWINDOWATTR));


			loadConfig(CFGFILE, &grabbers, &countGrabbers);
			for(i = 0; i < countGrabbers; i++)
				insertItemText(hwnd,CB_GRABBER,LIT_END,grabbers[i].id);
			selectItem(hwnd,CB_GRABBER,0);

			setText(hwnd, EF_BASENAME, "Track");

			/* wohw, this is too powerful, need cooling */

			len = sizeof(CDInfo);
			if(!DosOpen("\\DEV\\CD-ROM2$", &CDDevice, &action, 0,
							FILE_NORMAL, OPEN_ACTION_OPEN_IF_EXISTS,
							OPEN_SHARE_DENYNONE | OPEN_ACCESS_READONLY, NULL))
			{
				if(!DosDevIOCtl(CDDevice, 0x82, 0x60, NULL, 0, NULL, &CDInfo, len, &len))
				{
					for(i = 0; i < CDInfo.CountCD; i++)
					{
						char driveLetter[3] = { (char) ('A' + CDInfo.FirstCD + i), ':', 0};
						insertItemText(hwnd,CB_DRIVE,LIT_END,driveLetter);
					}
				}
				DosClose(CDDevice);
			}

			selectItem(hwnd,CB_DRIVE,0);

         wpCT = WinSubclassWindow(WinWindowFromID(hwnd,CT_TRACK),wpCTTrack);

         dataIco = WinLoadPointer(HWND_DESKTOP, NULLHANDLE, ICO_DATA);
         trackIco = WinLoadPointer(HWND_DESKTOP, NULLHANDLE, ICO_TRACK);
         loadIni(hwnd);
			return 0;
		}
		case WM_COMMAND:
			return processCommand(hwnd,mp1,mp2);
		case WM_CONTROL:
			return processControl(hwnd,mp1,mp2);
		case WM_ADJUSTFRAMEPOS:
		{
			SWP *pos = PVOIDFROMMP(mp1);
         static int bitRateCheck = 0;
         if(pos->fl & SWP_SIZE)
			{
				SWP ctpos;
				WinQueryWindowPos(WinWindowFromID(hwnd, CT_TRACK), &ctpos);
				WinSetWindowPos  (WinWindowFromID(hwnd, CT_TRACK), 0, 0, ctpos.y,
									  pos->cx, pos->cy - ctpos.y,
									  SWP_SIZE | SWP_SHOW | SWP_MOVE);
			}

         if((pos->fl & SWP_SHOW) && bitRateChanged != bitRateCheck)
			{
            bitRateCheck = bitRateChanged;
            refreshFieldInfo(hwnd, CT_TRACK);
			}
			break;
		}

      case WM_MENUEND:
         removeSourceEmphasis(HWNDFROMMP(mp2),&sourceEmphasisInfo);
			return 0;

      case WM_CLOSE:
         WinDestroyPointer(dataIco);
         WinDestroyPointer(trackIco);
         free(grabbers);
			saveIni(hwnd);
			return 0;

		case WM_CHAR:
			if(SHORT2FROMMP(mp2) == VK_ESC)
				return 0;
			else
				break;
	}

	return WinDefDlgProc( hwnd, msg, mp1, mp2 );
}
