/* CD2MP3 PM 1.0 (C) 1998 Samuel Audet <guardia@cam.org> */

#define INCL_DOSDEVIOCTL
#define INCL_DOS
#include <os2.h>
#include <stdio.h>

#include "readcd.h"

_Inline ULONG getHSG(MSF input)
{
	return (input.minute * 4500 + input.second * 75 + input.frame - 150);
}

_Inline MSF getMSF(ULONG input)
{
	MSF output;

	input += 150;
	output.frame = input % 75;
	input /= 75;
	output.second = input % 60;
	output.minute = input / 60;

	return output;
}

HFILE openCD(char *drive)
{
	HFILE hCDDrive = 0;
	ULONG ulAction;
	ULONG rc;

	rc = DosOpen(drive, &hCDDrive, &ulAction, 0,
					 FILE_NORMAL, OPEN_ACTION_OPEN_IF_EXISTS,
					 OPEN_SHARE_DENYNONE | OPEN_ACCESS_READONLY |
					 OPEN_FLAGS_DASD, NULL);
	if(rc)
		printf("DosOpen failed with return code %d\n",rc);

	return hCDDrive;
}

ULONG closeCD(HFILE hCDDrive)
{
	return DosClose(hCDDrive);
}

BOOL getCDInfo(HFILE hCDDrive, CDAUDIODISKINFODATA *cdInfo)
{
	ULONG ulParamLen;
	ULONG ulDataLen;
	ULONG rc;
	BOOL returnBool = FALSE;

	rc = DosDevIOCtl(hCDDrive, IOCTL_CDROMAUDIO, CDROMAUDIO_GETAUDIODISK,
						  "CD01", 4, &ulParamLen, cdInfo,
						  sizeof(*cdInfo), &ulDataLen);
	if(rc)
		printf("DosDevIOCtl failed with return code %d\n",rc);
	else
		returnBool = TRUE;

	return returnBool;
}

BOOL getTrackInfo(HFILE hCDDrive, char track, CDTRACKINFO *trackInfo2)
{
	ULONG ulParamLen;
	ULONG ulDataLen;
	ULONG rc;
	BOOL returnBool = FALSE;

	CDAUDIODISKINFODATA	 cdInfo;
	CDAUDIOTRACKINFOPARAM trackParam;
	CDAUDIOTRACKINFODATA  trackInfo[2];

	rc = DosDevIOCtl(hCDDrive, IOCTL_CDROMAUDIO, CDROMAUDIO_GETAUDIODISK,
						  "CD01", 4, &ulParamLen, &cdInfo,
						  sizeof(cdInfo), &ulDataLen);
	if(rc)
		printf("DosDevIOCtl failed with return code %d\n",rc);
	else
	{
		trackParam.signature[0] = 'C';
		trackParam.signature[1] = 'D';
		trackParam.signature[2] = '0';
		trackParam.signature[3] = '1';

		trackParam.trackNum = track;
		rc = DosDevIOCtl(hCDDrive, IOCTL_CDROMAUDIO, CDROMAUDIO_GETAUDIOTRACK,
							  &trackParam, sizeof(trackParam),
							  &ulParamLen, &trackInfo[0],
							  sizeof(trackInfo[0]), &ulDataLen);
		if(rc)
			printf("DosDevIOCtl failed with return code %d\n",rc);
		else
		{
			trackParam.trackNum = track+1;
			rc = 0;
			if(trackParam.trackNum <= cdInfo.lastTrack)
			{
				rc = DosDevIOCtl(hCDDrive, IOCTL_CDROMAUDIO, CDROMAUDIO_GETAUDIOTRACK,
									  &trackParam, sizeof(trackParam),
									  &ulParamLen, &trackInfo[1],
									  sizeof(trackInfo[1]), &ulDataLen);

				if(rc)
					printf("DosDevIOCtl failed with return code %d\n",rc);
			}
			else
				trackInfo[1].address = cdInfo.leadOutAddress;

			if(!rc)
			{
				ULONG HSG[2];
				MSF length;

				HSG[0] = getHSG(trackInfo[0].address);
				HSG[1] = getHSG(trackInfo[1].address);

				/* -150 because we want length, not an address */
				length = getMSF(HSG[1]-HSG[0]-150);

				trackInfo2->length = length;
				trackInfo2->size = (HSG[1]-HSG[0])*2352;
				trackInfo2->data = (trackInfo[0].info & 0x40) ? TRUE : FALSE;
				trackInfo2->channels = (trackInfo[0].info & 0x80) ? 4 : 2;
				trackInfo2->start = trackInfo[0].address;
				trackInfo2->end = trackInfo[1].address;

				returnBool = TRUE;
			}
		}
	}
	return returnBool;
}

BOOL playTrack(HFILE hCDDrive, char track)
{
	ULONG ulParamLen;
	ULONG ulDataLen;
	ULONG rc;
	BOOL returnBool = FALSE;
	CDPLAYAUDIOPARAM playParam;
	CDTRACKINFO trackInfo;

	if(getTrackInfo(hCDDrive, track, &trackInfo))
	{
		playParam.signature[0] = 'C';
		playParam.signature[1] = 'D';
		playParam.signature[2] = '0';
		playParam.signature[3] = '1';
		playParam.addressingMode = 1;
		playParam.start = trackInfo.start;
		playParam.end = trackInfo.end;

		rc = DosDevIOCtl(hCDDrive, IOCTL_CDROMAUDIO, CDROMAUDIO_PLAYAUDIO,
							  &playParam, sizeof(playParam),
							  &ulParamLen, NULL,
							  0, &ulDataLen);
		if(rc)
			printf("DosDevIOCtl failed with return code %d\n",rc);
		else
			returnBool = TRUE;
	}

	return returnBool;
}

BOOL stopCD(HFILE hCDDrive)
{
	ULONG ulParamLen;
	ULONG ulDataLen;
	ULONG rc;
	BOOL returnBool = FALSE;

	rc = DosDevIOCtl(hCDDrive, IOCTL_CDROMAUDIO, CDROMAUDIO_STOPAUDIO,
						  "CD01", 4, &ulParamLen, NULL, 0, &ulDataLen);
	if(rc)
		printf("DosDevIOCtl failed with return code %d\n",rc);
	else
		returnBool = TRUE;
	return returnBool;
}
